import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.net.CookieHandler;
import java.net.CookieManager;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

public class WebApiSample {
	/** URLベース */
    private static final String URL_BASE = "https://XXXXXXXX.spa-cloud.com/spa/service";

    /** URL ログイン */
    private static final String URL_LOGIN = URL_BASE + "/auth/login";
    /** URL 文書リスト */
    private static final String URL_DOCUMENT_LIST = URL_BASE + "/documents_v11/" + "16120" + "/list";
    /** URL ログアウト */
    private static final String URL_LOGOUT = URL_BASE + "/auth/logout";

    /** ユーザID */
    private static final String USERID = "admin";
    /** パスワード */
    private static final String PASSWORD = "admin";


    /**
     * main
     * @param args
     * @throws Exception
     */
	public static void main(String[] args) throws Exception {

        //全てのCookie
        String cookieAll = null;
        //JSESSIONID
        String jsessionId = null;
        //XSRF-TOKEN
        String xsrfToken = null;


        //Cookieの処理
		CookieManager cm = new CookieManager();
		CookieHandler.setDefault(cm);

        // HttpURLConnectionの作成
        URL urlLogin = new URL(URL_LOGIN);
        HttpURLConnection conn = (HttpURLConnection) urlLogin.openConnection();

        try {
            // HttpURLConnectionの各種設定
            conn.setRequestMethod("POST");
            conn.setDoInput(true);
            conn.setDoOutput(true);
            conn.setUseCaches(false);
            conn.setAllowUserInteraction(false);
            conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");

            // ユーザーIDとパスワードをセットする
            try (BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(conn.getOutputStream(), "UTF-8"))) {
                writer.write("user=" + USERID);
                writer.write("&password=" + PASSWORD);
            }

            // 正常に終了することができなかった場合
            if (conn.getResponseCode() != 200) {
                Map<String, List<String>> headers = conn.getHeaderFields();
            	StringBuffer hedaerBuf = new StringBuffer();
                for (Map.Entry<String, List<String>> entry:headers.entrySet()) {
                	String key = entry.getKey();
                	hedaerBuf.append(key + " : ");
                	List<String> list = entry.getValue();
                    for(Iterator<String>itr = list.iterator(); itr.hasNext();){
                    	String cookie = itr.next();
                    	hedaerBuf.append(cookie).append("; \r\n");
                    }
                }

                // HTTPステータスとSPA独自のエラーコードを取得できる
                int errCode = conn.getHeaderFieldInt("X-Spa-Error-Code", -1);
                System.out.println("HTTP ErrorCode[" + conn.getResponseCode() + "]");
                System.out.println("SPA ErrorCode[" + errCode + "]");
                String errMsg = conn.getHeaderField("X-Spa-Error-Message");
                if (errMsg != null) {
                    System.out.println("SPA Error Message[" + URLDecoder.decode(errMsg, "UTF-8") + "]");
                }
                throw new Exception(conn.getResponseMessage());
            }
            System.out.println("ログイン成功");

            // 正常終了時
            // セッションIDを取り出す（次のリクエスト以降で常にセット）
            Map<String, List<String>> headers = conn.getHeaderFields();
        	StringBuffer sbCookie = new StringBuffer();
            for (Map.Entry<String, List<String>> entry:headers.entrySet()) {
            	String key = entry.getKey();
            	//レスポンスヘッダからCookieを取り出す
            	if (key != null && key.equals("Set-Cookie")) {
            		List<String> list = entry.getValue();
                    for(Iterator<String>itr = list.iterator(); itr.hasNext();){
                    	String cookie = itr.next();
                    	sbCookie.append(cookie).append("; ");
                        String[] array = cookie.split(";");
                        for (String str : array) {
                        	//JSESSIONIDの値を取り出す
                        	if (str.matches("JSESSIONID=.*")) {
                            	String[] kvp = str.split("=");
                            	jsessionId = kvp[1];
                        	}
                        	//XSRF-TOKENの値を取り出す
                            if (str.matches("XSRF-TOKEN=.*")) {
                            	String[] kvp = str.split("=");
                            	xsrfToken = kvp[1];
                            }
                        }

                    }
                }
            }
            cookieAll = sbCookie.toString();
            System.out.println("cookieAll = " + cookieAll);
            System.out.println("JSESSIONID = " + jsessionId);
            System.out.println("XSRF-TOKEN = " + xsrfToken);
        } finally {
            // コネクションの切断
            conn.disconnect();
        }

        //サンプル実装　ログイン後のリクエスト処理
        //指定したフォルダID直下のファイルの一覧名称/IDを取得する
        // HttpURLConnectionの作成
        URL urlDocList = new URL(URL_DOCUMENT_LIST);
        conn = (HttpURLConnection) urlDocList.openConnection();

        List<String> documentIds = new ArrayList<String>();

        try {
            // HttpURLConnectionの各種設定
            conn.setRequestMethod("POST");
            conn.setDoInput(true);
            conn.setDoOutput(true);
            conn.setUseCaches(false);
            conn.setAllowUserInteraction(false);
            conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");

            // XSRF-TOKENとX-Requested-Withをセット(リクエストのヘッダに)
            conn.setRequestProperty("x-xsrf-token", xsrfToken);
            conn.setRequestProperty("x-requested-with", "XMLHttpRequest");

            // JSON形式で出力する指示
            conn.setRequestProperty("Accept", "application/json");

            // 正常に終了することができなかった場合
            if (conn.getResponseCode() != 200) {
                // HTTPステータスとSPA独自のエラーコードを取得できる
                int errCode = conn.getHeaderFieldInt("X-Spa-Error-Code", -1);
                System.out.println("HTTP ErrorCode[" + conn.getResponseCode() + "]");
                System.out.println("SPA ErrorCode[" + errCode + "]");
                String errMsg = conn.getHeaderField("X-Spa-Error-Message");
                if (errMsg != null) {
                    System.out.println("SPA Error Message[" + URLDecoder.decode(errMsg, "UTF-8") + "]");
                }
                throw new Exception(conn.getResponseMessage());
            }

            // 正常終了時
            // 正常終了した場合はJSONを取得できる　(jacksonライブラリを利用した場合)
            ObjectMapper mapper = new ObjectMapper();
            JsonNode document = mapper.readTree(conn.getInputStream());
            //ファイル名取得
            for (JsonNode n : document.get("documentList")) {
                if (n.get("type").asText().equals("document"))  {
                    String docid = n.get("id").asText();
                    documentIds.add(docid);
                    System.out.println("documentid =" + docid);
                }
            }

        } finally {
            conn.disconnect();
        }



        //サンプル実装　ログアウト
        // HttpURLConnectionの作成
        URL urlLogout = new URL(URL_LOGOUT);
        conn = (HttpURLConnection) urlLogout.openConnection();

        try {
            // HttpURLConnectionの各種設定
            conn.setRequestMethod("POST");
            conn.setDoInput(true);
            conn.setDoOutput(true);
            conn.setUseCaches(false);
            conn.setAllowUserInteraction(false);

            // XSRF-TOKENとX-Requested-Withをセット(リクエストのヘッダに)
            conn.setRequestProperty("x-xsrf-token", xsrfToken);
            conn.setRequestProperty("x-requested-with", "XMLHttpRequest");

            // 正常に終了することができなかった場合
            if (conn.getResponseCode() != 200) {
                // HTTPステータスとSPA独自のエラーコードを取得できる
                int errCode = conn.getHeaderFieldInt("X-Spa-Error-Code", -1);
                System.out.println("HTTP ErrorCode[" + conn.getResponseCode() + "]");
                System.out.println("SPA ErrorCode[" + errCode + "]");
                String errMsg = conn.getHeaderField("X-Spa-Error-Message");
                if (errMsg != null) {
                    System.out.println("SPA Error Message[" + URLDecoder.decode(errMsg, "UTF-8") + "]");
                }
                throw new Exception(conn.getResponseMessage());
            }
            System.out.println("logout succeed.");
        } finally {
            // コネクションの切断
            conn.disconnect();
        }
	}

}
